#include <Wire.h>
#include <Adafruit_TCS34725.h>

#define LedUV 9

const byte LedTCS = 7;
double dblSoma = 0.0;
double dblTotalGeral = 0.0;
int i, j, k = 0;
int incomingByte = 0;
int QtdLeituras = 0;
int LoopInterno = 10;
int LoopExterno = 50;

/* Initializes color sensor */
Adafruit_TCS34725 tcs = Adafruit_TCS34725(TCS34725_INTEGRATIONTIME_24MS, TCS34725_GAIN_1X);

void setup() {
  Serial.begin(9600);
  pinMode(LedTCS, OUTPUT);
  digitalWrite(LedTCS, LOW);
  pinMode(LedUV, OUTPUT);
  digitalWrite(LedUV, HIGH);


  /*--[Test the color sensor]---------------------------------*/

  Serial.println("--------------[COLOR SENSOR]--------------");
  Serial.println(" ");
  Serial.println("----[COLOR SENSOR TEST]----------------------------------------------");

  if (tcs.begin()) {
    Serial.println("  >> Color sensor OK.");
    Serial.println("----------------------------------------------------------------------------");
  } else {
    Serial.println("  >> Color sensor disconnected! Check the connections.");
    Serial.println("----------------------------------------------------------------------------");
    while (1)
      ;
  }

  /*--[Asks if you want to turn on the color sensor led]---------------------------------*/
  Serial.println(" ");
  Serial.println("Turning on the color sensor lighting? [Y/N]");

  while (!Serial.available()) {
    // Does nothing...
  }

  if (Serial.available() > 0) {
    incomingByte = Serial.read();
  }

  if (incomingByte == 'Y') {
    digitalWrite(LedTCS, HIGH);
    Serial.println("  >> Color sensor LED ON");
  } else if (incomingByte == 'N') {
    digitalWrite(LedTCS, LOW);
    Serial.println("  >> Color sensor LED OFF");
  }

  /*--[Asks if you want to adjust the intensity value of the UV LED]----------------------------*/
  Serial.println(" ");
  Serial.println("UV LED intensity [Integers from: 0 (off) to 9 (maximum)]");

  analogWrite(LedUV, 255);  // Initially, the LED is at maximum intensity;

  while (!Serial.available()) {
    // Does nothing...
  }

  if (Serial.available() > 0) {
    incomingByte = Serial.read();
  }

  if (incomingByte == '0') {
    analogWrite(LedUV, 0);
    Serial.println("  >> Intensity: OFF ");
  } else if (incomingByte == '1') {
    analogWrite(LedUV, 26);
    Serial.println("  >> Intensity: 10% ");
  } else if (incomingByte == '2') {
    analogWrite(LedUV, 51);
    Serial.println("  >> Intensity: 20% ");
  } else if (incomingByte == '3') {
    analogWrite(LedUV, 77);
    Serial.println("  >> Intensity: 30% ");
  } else if (incomingByte == '4') {
    analogWrite(LedUV, 102);
    Serial.println("  >> Intensity: 40% ");
  } else if (incomingByte == '5') {
    analogWrite(LedUV, 128);
    Serial.println("  >> Intensity: 50% ");
  } else if (incomingByte == '6') {
    analogWrite(LedUV, 153);
    Serial.println("  >> Intensity: 60% ");
  } else if (incomingByte == '7') {
    analogWrite(LedUV, 179);
    Serial.println("  >> Intensity: 70% ");
  } else if (incomingByte == '8') {
    analogWrite(LedUV, 204);
    Serial.println("  >> Intensity: 80% ");
  } else if (incomingByte == '9') {
    analogWrite(LedUV, 255);
    Serial.println("  >> Intensity: 100% ");
  }
  Serial.println(" ");
  Serial.println("Waiting for the UV LED to stabilize...");
  delay(3000);

  Serial.println(" ");
  Serial.println("--------------[INITIALIZING SIGNAL ANALYSIS]---------------------------");
}

void loop() {

  /*--[Ask the number of analyses to be carried out]---------------------------------*/

  Serial.println(" ");
  Serial.println("How many readings do you want to take? [Enter integer value]");

  while (!Serial.available()) {
    // Does nothing...
  }

  if (Serial.available() > 0) {
    QtdLeituras = Serial.parseInt();
    while (QtdLeituras == 0) {
      // Expect a value greater than zero;
    }
  }

  Serial.print("Total readings: ");
  Serial.println(QtdLeituras);
  Serial.println(" ");

  for (k = 0; k < QtdLeituras; k++) {
    //-------------------------------------------------------xx

    /*--[Creates a variable to be used by the TCS34725 and reads the color values]-----*/

    uint16_t r, g, b, c, colorTemp, lux;
    tcs.getRawData(&r, &g, &b, &c);

    Serial.print("R: ");
    Serial.print(r, DEC);
    Serial.print(" ");
    Serial.print("G: ");
    Serial.print(g, DEC);
    Serial.print(" ");
    Serial.print("B: ");
    Serial.print(b, DEC);
    Serial.print(" ");
    Serial.print("C: ");
    Serial.println(c, DEC);

    //-------------------------------------------------------xx
  }

  /*--[Ask if you want to do more analysis]-------------------------------------------------------*/

  Serial.println(" ");
  Serial.println("Would you like to do another analysis? [Y/N]");

  while (!Serial.available()) {
    // Does nothing...
  }

  if (Serial.available() > 0) {
    incomingByte = Serial.read();
  }

  if (incomingByte == 'N') {
    Serial.println(" ");
    Serial.println(" ");
    Serial.println(">>> Done! You can disconnect the analyzer");
    while (true)
      ;
  }
}
